#include <SensirionI2cScd30.h> // Library for the Sensirion SCD30 CO2 sensor
#include "SparkFun_SCD30_Arduino_Library.h" // SparkFun SCD30 library
#include "SparkFunCCS811.h" // SparkFun CCS811 air quality sensor library
#include <U8g2lib.h> // Library for OLED display
#include "DHT.h" // DHT sensor library for temperature and humidity
#include <WiFi.h> // WiFi library for ESP32
#include <ESPmDNS.h> // mDNS library for ESP32
#include <WebServer.h> // Web server library for ESP32
#include "ThingSpeak.h" // ThingSpeak library for IoT data logging

// Define sensor types and pins

#define DHTTYPE DHT22 // DHT 22 (AM2302), specify the type of DHT sensor

// DHT Sensor setup
uint8_t DHTPin = 4; // GPIO pin where the DHT sensor is connected
               
// Initialize DHT sensor with the specified pin and type
DHT dht(DHTPin, DHTTYPE);

#define CCS811_ADDR 0x5A // 0x5B // I2C address for the CCS811 sensor, alternative address could be 0x5B

CCS811 mySensor(CCS811_ADDR); // Initialize CCS811 sensor with its I2C address

// Initialize SCD30 sensor
SCD30 airSensor;

static char errorMessage[128]; // Buffer to hold error messages
static int16_t error; // Variable to hold error codes

const char* ssid = "...";  // Enter SSID here
const char* password = "...";  // Enter Password here

// Variables to hold sensor readings
float Temperature;
float Humidity;
float co2;
float tvoc;

// Variables to hold integer sensor readings
int CO2;
int TVOC;

// Web server setup
WebServer server(80); // Initialize web server on port 80
WiFiClient  client;  // Create a WiFi client instance

// ThingSpeak setup
unsigned long myChannelNumber = 1; // ThingSpeak channel number
const char * myWriteAPIKey = "3V0PGII1Y1D4833R"; // ThingSpeak Write API Key

// Timing variables for updates
unsigned long lastTime = 0;
unsigned long timerDelay = 30000; // Delay between ThingSpeak updates, in milliseconds

unsigned long lastSensorUpdateTime = 0; // Last time sensors were updated
const unsigned long sensorUpdateInterval = 2000; // Sensor update interval in milliseconds
int lastValidTVOC = -1; // Store the last valid TVOC reading
int lastValidCO2 = -1; // Store the last valid CO2 reading


// Define buttons and screen mode
#define Allsensor_Screen 2
#define Temperature_Button 15
#define Humidity_Button 12
#define TVOC_Button 13
#define CO2_Button 14

// Include SPI and Wire libraries if hardware SPI and I2C are available
#ifdef U8X8_HAVE_HW_SPI
#include <SPI.h>
#endif
#ifdef U8X8_HAVE_HW_I2C
#include <Wire.h>
#endif

// Icons and images for the display, stored in program memory
static const unsigned char image_weather_temperature_bits[] U8X8_PROGMEM = {0x38,0x00,0x44,0x40,0xd4,0xa0,0x54,0x40,0xd4,0x1c,0x54,0x06,0xd4,0x02,0x54,0x02,0x54,0x06,0x92,0x1c,0x39,0x01,0x75,0x01,0x7d,0x01,0x39,0x01,0x82,0x00,0x7c,0x00};
static const unsigned char image_cloud_bits[] U8X8_PROGMEM = {0x00,0x00,0x00,0xe0,0x03,0x00,0x10,0x04,0x00,0x08,0x08,0x00,0x0c,0x10,0x00,0x02,0x70,0x00,0x01,0x80,0x00,0x01,0x00,0x01,0x02,0x00,0x01,0xfc,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
static const unsigned char image_operation_warning_bits[] U8X8_PROGMEM = {0x00,0x00,0x80,0x01,0x40,0x02,0x40,0x02,0x20,0x04,0x90,0x09,0x90,0x09,0x88,0x11,0x88,0x11,0x84,0x21,0x02,0x40,0x82,0x41,0x81,0x81,0x01,0x80,0xfe,0x7f,0x00,0x00};
static const unsigned char image_Layer_6_bits[] U8X8_PROGMEM = {0xf0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x03,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0xf0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x03};
static const unsigned char image_network_3_bars_bits[] U8X8_PROGMEM = {0x00,0x70,0x00,0x50,0x00,0x50,0x00,0x50,0x00,0x57,0x00,0x57,0x00,0x57,0x00,0x57,0x70,0x57,0x70,0x57,0x70,0x57,0x70,0x57,0x77,0x57,0x77,0x57,0x77,0x77,0x00,0x00};
static const unsigned char image_Layer_7_bits[] U8X8_PROGMEM = {0xe0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x07,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x40,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x40,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x40,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x40,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0xe0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x07};
static const unsigned char image_weather_humidity_bits[] U8X8_PROGMEM = {0x20,0x00,0x20,0x00,0x30,0x00,0x70,0x00,0x78,0x00,0xf8,0x00,0xfc,0x01,0xfc,0x01,0x7e,0x03,0xfe,0x02,0xff,0x06,0xff,0x07,0xfe,0x03,0xfe,0x03,0xfc,0x01,0xf0,0x00};
static const unsigned char image_Layer_9_bits[] U8X8_PROGMEM = {0x1e,0x38,0x31,0x3c,0x3f,0x1e,0xb7,0x0f,0xfe,0x07,0xfe,0x07,0xf4,0x01,0xf8,0x00,0x7e,0x3f,0xbe,0x3f,0x9f,0x3f,0x8f,0x3f,0x07,0x3f,0x00,0x0e};
static const unsigned char image_cloud_download_bits[] U8X8_PROGMEM = {0x00,0x00,0x00,0xe0,0x03,0x00,0x10,0x04,0x00,0x08,0x08,0x00,0x0c,0x10,0x00,0x02,0x70,0x00,0x01,0x80,0x00,0x01,0x00,0x01,0x02,0x01,0x01,0x3c,0xf9,0x00,0x00,0x01,0x00,0x00,0x01,0x00,0xc0,0x07,0x00,0x80,0x03,0x00,0x00,0x01,0x00,0x00,0x00,0x00};
// Additional icons omitted for brevity...

// Variables to control display screens
bool showAllSensorScreen = true;
bool showHumidityScreen = false;
bool showCO2Screen = false;
bool showTVOCScreen = false;
bool showTempScreen = false;

// Variables to track highest and lowest values of sensors
float highTemp = -100;
float lowTemp = 100;
float highHumidity = -2;
float lowHumidity = 101;
int lowCO2 = 2000;
int highCO2 = 100;
int lowTVOC = 200;
int highTVOC = 20;

  // Initialize the OLED display (model, rotation, MOSI, CLK, CS, DC, RESET)
  U8G2_SSD1309_128X64_NONAME2_F_4W_HW_SPI u8g2(U8G2_R0, 5, 16, 17);
  

void draw_all(void) {
  // Function to redraw the complete screen with all sensor readings
  // Read humidity
  float h = dht.readHumidity();
  // Read temperature as Celsius
  float t = dht.readTemperature();
  // Read temperature as Fahrenheit (unused in this function but useful for conditional rendering or logging)
  float f = dht.readTemperature(true);

  // Update high and low temperatures
  if (t > highTemp) {
    highTemp = t;
  }

  if (t < lowTemp) {
    lowTemp = t;
  }

  int temp = dht.readTemperature();

  // Setting color index for OLED display drawing
  u8g2.setColorIndex(1);
  // Setting font for text display
  u8g2.setFont(u8g2_font_helvB08_tr);
  // Displaying Temperature value
  u8g2.setCursor(20, 13); 
  u8g2.print("Temperature : "); u8g2.print(t); u8g2.print("C");
  // Displaying Humidity value
  u8g2.setCursor(22, 29);
  u8g2.print("Humidity : "); u8g2.print(h); u8g2.print("%");
  // Displaying CO2 value
  u8g2.setCursor(22, 45);
  u8g2.print("CO2 : "); u8g2.print(CO2); u8g2.print("ppm");
  // Displaying TVOC value
  u8g2.setCursor(22, 61);
  u8g2.print("TVOC : "); u8g2.print(TVOC); u8g2.print("ppb");

  // Drawing icons next to the sensor values
  u8g2.drawXBMP(1, 2, 16, 16, image_weather_temperature_bits);
  u8g2.drawXBMP(1, 19, 17, 16, image_cloud_bits);
  u8g2.drawXBMP(1, 33, 17, 16, image_cloud_download_bits);
  u8g2.drawXBMP(1, 47, 15, 16, image_network_3_bars_bits);
}

void draw_temp(void) {
  // Read humidity as scale from 0 - 100%
  //float h = dht.readHumidity();
  // Read temperature as Celsius
  float t = dht.readTemperature();
  // Read temperature as Fahrenheit
  //float f = dht.readTemperature(true);
  // Find Max Temp
  if (t > highTemp) {
    highTemp = t;
  }

  // Find Min Temp
  if (t < lowTemp) {
    lowTemp = t;
  }

  int temp_bar_fill;

  if (t == highTemp) {
    temp_bar_fill = 100;
  } else if (t == lowTemp) {
    temp_bar_fill = 10;
  } else {
    // Calculate the percentage of the current temperature between lowTemp and highTemp
    float percentage = float(t - lowTemp) / float(highTemp - lowTemp) * 100.0;
    // Ensure the percentage is within the range [0, 100]
    percentage = constrain(percentage, 0, 100);
    // Convert the percentage to an integer for temp_bar_fill
    temp_bar_fill = int(percentage);
  }
  
  
  //Print Max Temp
  u8g2.setBitmapMode(1);
  u8g2.setFontMode(1);
  u8g2.setFont(u8g2_font_helvB08_tr);
  u8g2.setCursor(3, 40);
  u8g2.print("High : "); u8g2.print(highTemp); u8g2.print("C");
  
  //Print Min Temp
  u8g2.setFont(u8g2_font_helvB08_tr);
  u8g2.setCursor(3, 28);
  u8g2.print("Low : "); u8g2.print(lowTemp); u8g2.print("C");
  
  //Print Current Temp
  u8g2.setFont(u8g2_font_profont22_tr);
  u8g2.setCursor(23, 59);
  u8g2.print(t); u8g2.print("C");

  // Icons Code Placement
  u8g2.drawXBMP(3, 43, 16, 16, image_weather_temperature_bits);

  // Draw frame and box
  u8g2.drawXBMP(2, 2, 104, 15, image_Layer_7_bits);
  u8g2.drawRBox(4, 4, temp_bar_fill, 11, 4);

  // If temp is 35.00°C or above a warning icon will be shown
  if (t >= 35.00 || t <= 10.00) {
    u8g2.drawXBMP(99, 44, 16, 16, image_operation_warning_bits);    
  } else {
    return;
  }
}

void draw_humidity(void) {
  
  float h = dht.readHumidity();

  // Find Highest Humidity
  if (h > highHumidity) {
    highHumidity = h;
  }

  // Find Lowest Humidity
  if (h < lowHumidity) {
    lowHumidity = h;
  }

  int humidity_bar_fill = map(h, lowHumidity, highHumidity, 0, 100);

  humidity_bar_fill = constrain(humidity_bar_fill, 0, 100);

  if (h == highHumidity && h == lowHumidity) {
    humidity_bar_fill == 100;
  } 

  if (h == highHumidity) {
    humidity_bar_fill == 100;
  } else if (h == lowHumidity) {
    humidity_bar_fill == 10;
  }


  u8g2.setBitmapMode(1);
  u8g2.setFontMode(1);
  u8g2.setFont(u8g2_font_helvB08_tr);
  u8g2.setCursor(3, 28);
  u8g2.print("Low : "); u8g2.print(lowHumidity); u8g2.print("%");
  
  u8g2.setFont(u8g2_font_helvB08_tr);
  u8g2.setCursor(3, 40);
  u8g2.print("High : "); u8g2.print(highHumidity); u8g2.print("%");

  u8g2.setFont(u8g2_font_profont22_tr);
  u8g2.setCursor(19, 60);
  u8g2.print(h);

  // Print Humidity Icon
  u8g2.drawXBMP(4, 43, 11, 16, image_weather_humidity_bits);
  u8g2.drawXBMP(79, 46, 14, 14, image_Layer_9_bits);

  u8g2.drawXBMP(2, 2, 104, 15, image_Layer_7_bits);
  u8g2.drawRBox(4, 4, humidity_bar_fill, 11, 4);

  if (h >= 60.00 || h <= 40.00) {
    u8g2.drawXBMP(99, 44, 16, 16, image_operation_warning_bits);    
  } else {
    return;
  }

}

void draw_CO2(void) {
  //int CO2 = airSensor.getCO2();

  // Read and print SCD30 data

  if (CO2 > highCO2) {
    highCO2 = CO2;
  }

  // Find Lowest Humidity
  if (CO2 < lowCO2) {
    lowCO2 = CO2;
  }

  int CO2_bar_fill;

  if (CO2 == highCO2) {
    CO2_bar_fill = 100;
  } else if (CO2 == lowCO2) {
    CO2_bar_fill = 10;
  } else {
    // Calculate the percentage of the current temperature between lowTemp and highTemp
    float percentage_1 = float(CO2 - lowCO2) / float(highCO2 - lowCO2) * 100.0;
    // Ensure the percentage is within the range [0, 100]
    percentage_1 = constrain(percentage_1, 0, 100);
    // Convert the percentage to an integer for temp_bar_fill
    CO2_bar_fill = int(percentage_1);
  }

  u8g2.setBitmapMode(1);
  u8g2.setFontMode(1);
  u8g2.setFont(u8g2_font_helvB08_tr);
  u8g2.setCursor(3, 28);
  u8g2.print("Low : "); u8g2.print(lowCO2); u8g2.print(" ppm");
  
  u8g2.setFont(u8g2_font_helvB08_tr);
  u8g2.setCursor(3, 40);
  u8g2.print("High : "); u8g2.print(highCO2); u8g2.print(" ppm");
  
  u8g2.setFont(u8g2_font_profont22_tr);
  u8g2.setCursor(24, 60);
  u8g2.print(CO2);

  if (CO2 >= 1000){
    u8g2.setFont(u8g2_font_helvB08_tr);
    u8g2.setCursor(74, 59);
    u8g2.print("ppm");
  } else if (CO2 < 1000) {
      u8g2.setFont(u8g2_font_helvB08_tr);
      u8g2.setCursor(63, 59);
      u8g2.print("ppm");
  }
  
  u8g2.drawXBMP(2, 2, 104, 15, image_Layer_7_bits);
  u8g2.drawRBox(4, 4, CO2_bar_fill, 11, 4);
  u8g2.drawXBMP(2, 45, 17, 16, image_cloud_download_bits); 

  // Check and display warning based on TVOC levels
  if (CO2 <= 400 || CO2 >= 900) { // Condition for displaying a warning
    if (CO2 <= 999) {
        // Display warning icon for TVOC values less than or equal to 999
        u8g2.drawXBMP(88, 45, 16, 16, image_operation_warning_bits);
    } else if (CO2 >= 1000) {
        // Display warning icon for TVOC values greater than or equal to 1000
        // Assuming you might want to change icon position/size for different TVOC levels
        u8g2.drawXBMP(98, 45, 16, 16, image_operation_warning_bits); // Example adjustment
    }
  } // No need for an else statement that returns; it's redundant outside a function

}

void draw_TVOC(void) {

  if (TVOC > highTVOC) {
    highTVOC = TVOC;
  }

  // Find Lowest Humidity
  if (TVOC < lowTVOC) {
    lowTVOC = TVOC;
  }

  int TVOC_bar_fill;

  if (TVOC == highTVOC) {
    TVOC_bar_fill = 100;
  } else if (TVOC == lowTVOC) {
    TVOC_bar_fill = 10;
  } else {
    // Calculate the percentage of the current temperature between lowTemp and highTemp
    float percentage_2 = float(TVOC - lowTVOC) / float(highTVOC - lowTVOC) * 100.0;
    // Ensure the percentage is within the range [0, 100]
    percentage_2 = constrain(percentage_2, 0, 100);
    // Convert the percentage to an integer for temp_bar_fill
    TVOC_bar_fill = int(percentage_2);
  }

  u8g2.setBitmapMode(1);
  u8g2.setFontMode(1);
  u8g2.setFont(u8g2_font_helvB08_tr);
  u8g2.setCursor(3, 28);
  u8g2.print("Low : "); u8g2.print(lowTVOC); u8g2.print(" ppb");
  
  u8g2.setFont(u8g2_font_helvB08_tr);
  u8g2.setCursor(3, 40);
  u8g2.print("High : "); u8g2.print(highTVOC); u8g2.print(" ppb");
  
  u8g2.setFont(u8g2_font_profont22_tr);
  u8g2.setCursor(24, 60);
  u8g2.print(TVOC);

  if (TVOC <= 999) {
    u8g2.setFont(u8g2_font_helvB08_tr);
    u8g2.setCursor(63, 59);
    u8g2.print("ppb");
  } else if (TVOC >= 1000) {
    u8g2.setFont(u8g2_font_helvB08_tr);
    u8g2.setCursor(73, 59);
    u8g2.print("ppb");
  }
  
  
  
  u8g2.drawXBMP(2, 2, 104, 15, image_Layer_7_bits);
  u8g2.drawRBox(4, 4, TVOC_bar_fill, 11, 4);

  u8g2.drawXBMP(3, 47, 17, 16, image_cloud_bits);  

  // Check and display warning based on TVOC levels
  if (TVOC <= 100 || TVOC >= 300) { // Condition for displaying a warning
    if (TVOC <= 999) {
        // Display warning icon for TVOC values less than or equal to 999
        u8g2.drawXBMP(85, 45, 16, 16, image_operation_warning_bits);
    } else if (TVOC >= 1000) {
        // Display warning icon for TVOC values greater than or equal to 1000
        // Assuming you might want to change icon position/size for different TVOC levels
        u8g2.drawXBMP(93, 45, 16, 16, image_operation_warning_bits); // Example adjustment
    }
  } // No need for an else statement that returns; it's redundant outside a function

}

void setup(void) {

  Serial.begin(115200);
  delay(100);

  dht.begin();

  u8g2.begin();  

  pinMode(Temperature_Button, INPUT_PULLUP);
  pinMode(Allsensor_Screen, INPUT_PULLUP);
  pinMode(Humidity_Button, INPUT_PULLUP);
  pinMode(TVOC_Button, INPUT_PULLUP);
  pinMode(CO2_Button, INPUT_PULLUP);

  Serial.println("Connecting to ");
  Serial.println(ssid);

  //connect to your local wi-fi network
  WiFi.begin(ssid, password);

  //check wi-fi is connected to wi-fi network
  while (WiFi.status() != WL_CONNECTED) {
  delay(1000);
  Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected..!");
  Serial.print("Got IP: ");  Serial.println(WiFi.localIP());

  server.on("/", handle_OnConnect);
  server.onNotFound(handle_NotFound);

  server.begin();
  Serial.println("HTTP server started");

  WiFi.mode(WIFI_STA);   
  
  ThingSpeak.begin(client);

  Serial.println("SCD30 Example");

  Wire.begin();

  if (mySensor.begin() == false) {
    Serial.println("CCS811 error. Please check wiring. Freezing...");
    //while (1);
  }

  // Initialize SCD30
  if (airSensor.begin() == false) {
    Serial.println("Air sensor not detected. Please check wiring. Freezing...");
    //while (1);
  }


}

void loop(void) {

  // Wait a few seconds between measurements.
  //delay(1000);
  server.handleClient();
  // Reading temperature or humidity takes about 250 milliseconds!
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
  float h = dht.readHumidity();
  // Read temperature as Celsius
  float t = dht.readTemperature();

  // Check if enough time has passed to update the sensor values
  if (millis() - lastSensorUpdateTime >= sensorUpdateInterval) {
    // It's time to read and store new data
    if (mySensor.dataAvailable()) {
      mySensor.readAlgorithmResults();
      int newTVOC = mySensor.getTVOC();
      if (newTVOC > 0) { // Check if TVOC reading is greater than 0
        TVOC = newTVOC;
        lastValidTVOC = TVOC; // Update the last valid TVOC
        Serial.print("CCS811 -> TVOC: ");
        Serial.println(TVOC);
      } else if (lastValidTVOC != -1) { // Use last valid TVOC if new reading is 0
        TVOC = lastValidTVOC;
        Serial.print("CCS811 -> TVOC (using last valid): ");
        Serial.println(TVOC);
      }
    }

    if (airSensor.dataAvailable()) {
      int newCO2 = airSensor.getCO2();
      if (newCO2 > 0) { // Check if CO2 reading is greater than 0
        CO2 = newCO2;
        lastValidCO2 = CO2; // Update the last valid CO2
        Serial.print("SCD30 -> CO2: ");
        Serial.println(CO2);
      } else if (lastValidCO2 != -1) { // Use last valid CO2 if new reading is 0
        CO2 = lastValidCO2;
        Serial.print("SCD30 -> CO2 (using last valid): ");
        Serial.println(CO2);
      }
    }

    lastSensorUpdateTime = millis(); // Update the last update time
  }

  if (t > highTemp) {
    highTemp = t;
  }
  
  if (t < lowTemp) {
    lowTemp = t;
  }

  if (t > highHumidity) {
    highHumidity = t;
  }

  // Find Lowest Humidity
  if (t < lowHumidity) {
    lowHumidity = t;
  }

  if (digitalRead(Allsensor_Screen) == LOW) {
    showAllSensorScreen = true;
    showHumidityScreen = false;
    showTVOCScreen = false;
    showCO2Screen = false;
    showTempScreen = false;
  } else if (digitalRead(Temperature_Button) == LOW) {
    showAllSensorScreen = false;
    showHumidityScreen = false; // Ensure the humidity screen is not shown
    showTVOCScreen = false;
    showCO2Screen = false;
    showTempScreen = true;
  } else if (digitalRead(Humidity_Button) == LOW) {
    showAllSensorScreen = false; // Ensure the temperature screen is not shown
    showHumidityScreen = true;
    showTVOCScreen = false;
    showCO2Screen = false;
    showTempScreen = false;
  } else if (digitalRead(TVOC_Button)== LOW) {
    showAllSensorScreen = false; 
    showHumidityScreen = false;
    showTVOCScreen = true;
    showCO2Screen = false;
    showTempScreen = false;
  } else if (digitalRead(CO2_Button) == LOW) {
    showAllSensorScreen = false; 
    showHumidityScreen = false;
    showTVOCScreen = false;
    showCO2Screen = true;
    showTempScreen = false;
  }

  if (showAllSensorScreen) {
    // Display the "All Sensor Screen"
    u8g2.firstPage();
    do {
        draw_all();
    } while (u8g2.nextPage());
  } else if (showHumidityScreen) {
    // Display the "Humidity Screen"
    u8g2.firstPage();
    do {
        draw_humidity();
    } while (u8g2.nextPage());
  } else if (showTempScreen) {
    // Display the "Temperature Screen"
    u8g2.firstPage();
    do {
        draw_temp();
    } while (u8g2.nextPage());
  } else if (showCO2Screen){
    u8g2.firstPage();
    do {
        draw_CO2();
    } while (u8g2.nextPage());
  } else if (showTVOCScreen){
  u8g2.firstPage();
    do {
        draw_TVOC();
    } while (u8g2.nextPage());
  }

  if ((millis() - lastTime) > timerDelay) {

  ThingSpeak.setField(1, t);
  ThingSpeak.setField(2, h);
  ThingSpeak.setField(3, CO2);
  ThingSpeak.setField(4, TVOC);

  int x = ThingSpeak.writeFields(myChannelNumber, myWriteAPIKey);

  if(x == 200){
      Serial.println("Channel update successful.");
    }
    else{
      Serial.println("Problem updating channel. HTTP error code " + String(x));
    }
    lastTime = millis();
  }

}

void handle_OnConnect() {

  Temperature = dht.readTemperature(); // Gets the values of the temperature
   Humidity = dht.readHumidity(); // Gets the values of the humidity 
    co2 = CO2;
     tvoc = TVOC;
  server.send(200, "text/html", SendHTML(Temperature,Humidity,co2,tvoc)); 
}

void handle_NotFound(){
  server.send(404, "text/plain", "Not found");
}

String SendHTML(float Temperaturestat, float Humiditystat, float co2stat, float tvocstat){
  String ptr = "<!DOCTYPE html> <html>\n";
  ptr +="<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=no\">\n";
  ptr +="<title>ESP32 Weather Station</title>\n";
  ptr +="<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n";
  ptr +="body{margin-top: 50px;} h1 {color: #444444;margin: 50px auto 30px;}\n";
  ptr +="p {font-size: 24px;color: #444444;margin-bottom: 10px;}\n";
  ptr +="</style>\n";
  ptr +="</head>\n";
  ptr +="<body>\n";
  ptr +="<div id=\"webpage\">\n";
  ptr +="<h1>ESP32 Weather Station</h1>\n";

  ptr +="<p>Temperature: ";
  ptr += (float)Temperaturestat;
  ptr +="&deg;C</p>";
  ptr +="<p>Humidity: ";
  ptr += (float)Humiditystat;
  ptr +="%</p>";
  ptr +="<p>CO2: ";
  ptr += (int)co2stat;
  ptr +="ppm</p>";
  ptr +="<p>TVOC: ";
  ptr += (int)tvocstat;
  ptr +="ppb</p>";

  ptr +="</div>\n";
  ptr +="</body>\n";
  ptr +="</html>\n";
  return ptr;
}



